@@ -26,6 +26,7 @@ from metomi.isodatetime.dumpers import TimePointDumper
 from metomi.isodatetime.timezone import (
     get_local_time_zone, get_local_time_zone_format, TimeZoneFormatMode)
 from metomi.isodatetime.exceptions import IsodatetimeError
+from metomi.isodatetime.parsers import ISO8601SyntaxError
 from cylc.flow.time_parser import CylcTimeParser
 from cylc.flow.cycling import (
     PointBase, IntervalBase, SequenceBase, ExclusionBase, cmp
@@ -34,7 +35,8 @@ from cylc.flow.exceptions import (
     CylcConfigError,
     IntervalParsingError,
     PointParsingError,
-    SequenceDegenerateError
+    SequenceDegenerateError,
+    WorkflowConfigError
 )
 from cylc.flow.wallclock import get_current_time_string
 from cylc.flow.parsec.validate import IllegalValueError
@@ -58,16 +60,16 @@ WARNING_PARSE_EXPANDED_YEAR_DIGITS = (
 class WorkflowSpecifics:
 
     """Store workflow-setup-specific constants and utilities here."""
-    ASSUMED_TIME_ZONE: Optional[Tuple[int, int]] = None
-    DUMP_FORMAT: Optional[str] = None
+    ASSUMED_TIME_ZONE: Tuple[int, int]
+    DUMP_FORMAT: str
+    abbrev_util: CylcTimeParser
+    interval_parser: 'DurationParser'
+    point_parser: 'TimePointParser'
+    recurrence_parser: 'TimeRecurrenceParser'
+    iso8601_parsers: Tuple[
+        'TimePointParser', 'DurationParser', 'TimeRecurrenceParser'
+    ]
     NUM_EXPANDED_YEAR_DIGITS: int = 0
-    abbrev_util: Optional[CylcTimeParser] = None
-    interval_parser: 'DurationParser' = None
-    point_parser: 'TimePointParser' = None
-    recurrence_parser: 'TimeRecurrenceParser' = None
-    iso8601_parsers: Optional[
-        Tuple['TimePointParser', 'DurationParser', 'TimeRecurrenceParser']
-    ] = None
 
 
 class ISO8601Point(PointBase):
@@ -685,10 +687,8 @@ def ingest_time(value: str, now: Optional[str] = None) -> str:
     if offset is not None:
         # add/subtract offset duration to/from chosen timepoint
         duration_parser = WorkflowSpecifics.interval_parser
-
         offset = offset.replace('+', '')
-        offset = duration_parser.parse(offset)
-        cycle_point += offset
+        cycle_point += duration_parser.parse(offset)
 
     return str(cycle_point)
 
@@ -739,7 +739,18 @@ def prev_next(
     }
 
     for my_time in str_points:
-        parsed_point = parser.parse(my_time.strip())
+        try:
+            parsed_point = parser.parse(my_time.strip())
+        except ISO8601SyntaxError as exc:
+            raise exc
+        except ValueError:
+            # If list is accidentally comma
+            suggest = my_time.replace(',', ';')
+            raise WorkflowConfigError(
+                f'Invalid offset: {my_time}:'
+                f' Offset lists are semicolon separated, try {suggest}'
+            )
+
         timepoints.append(parsed_point + now)
 
         if direction == 'previous':
