@@ -647,7 +647,7 @@ class Workflow(ObjectType):
         resolver=get_nodes_by_ids)
     jobs = graphene.List(
         lambda: Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
@@ -703,6 +703,58 @@ class Workflow(ObjectType):
     pruned = Boolean()
 
 
+class RuntimeSetting(ObjectType):
+    """Key = value setting for a `[runtime][<namespace>]` configuration."""
+    key = String(default_value=None)
+    value = String(default_value=None)
+
+
+class Runtime(ObjectType):
+    class Meta:
+        description = sstrip("""
+            Subset of runtime fields.
+
+            Existing on 3 different node types:
+            - Task/family definition (from the flow.cylc file)
+            - Task/family cycle instance (includes any broadcasts on top
+              of the definition)
+            - Job (a record of what was run for a particular submit)
+        """)
+    platform = String(default_value=None)
+    script = String(default_value=None)
+    init_script = String(default_value=None)
+    env_script = String(default_value=None)
+    err_script = String(default_value=None)
+    exit_script = String(default_value=None)
+    pre_script = String(default_value=None)
+    post_script = String(default_value=None)
+    work_sub_dir = String(default_value=None)
+    execution_polling_intervals = String(default_value=None)
+    execution_retry_delays = String(default_value=None)
+    execution_time_limit = String(default_value=None)
+    submission_polling_intervals = String(default_value=None)
+    submission_retry_delays = String(default_value=None)
+    directives = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+    environment = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+    outputs = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+
+
+RUNTIME_FIELD_TO_CFG_MAP = {
+    **{
+        k: k.replace('_', ' ') for k in Runtime.__dict__
+        if not k.startswith('_')
+    },
+    'init_script': 'init-script',
+    'env_script': 'env-script',
+    'err_script': 'err-script',
+    'exit_script': 'exit-script',
+    'pre_script': 'pre-script',
+    'post_script': 'post-script',
+    'work_sub_dir': 'work sub-directory',
+}
+"""Map GQL Runtime fields' names to workflow config setting names."""
+
+
 class Job(ObjectType):
     class Meta:
         description = """Jobs."""
@@ -724,22 +776,12 @@ class Job(ObjectType):
     finished_time = String()
     job_id = ID()
     job_runner_name = String()
-    env_script = String()
-    err_script = String()
-    exit_script = String()
     execution_time_limit = Float()
     platform = String()
-    init_script = String()
     job_log_dir = String()
-    post_script = String()
-    pre_script = String()
-    script = String()
-    work_sub_dir = String()
-    environment = GenericScalar(resolver=resolve_json_dump)
-    directives = GenericScalar(resolver=resolve_json_dump)
-    param_var = GenericScalar(resolver=resolve_json_dump)
     extra_logs = graphene.List(String)
     messages = graphene.List(String)
+    runtime = Field(Runtime)
 
 
 class Task(ObjectType):
@@ -748,6 +790,7 @@ class Task(ObjectType):
     id = ID()  # noqa: A003 (required for definition)
     name = String()
     meta = Field(NodeMeta)
+    runtime = Field(Runtime)
     mean_elapsed_time = Float()
     depth = Int()
     proxies = graphene.List(
@@ -855,6 +898,7 @@ class TaskProxy(ObjectType):
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
         resolver=get_node_by_id)
+    runtime = Field(Runtime)
     state = String()
     cycle_point = String()
     is_held = Boolean()
@@ -899,7 +943,7 @@ class TaskProxy(ObjectType):
     prerequisites = graphene.List(Prerequisite)
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
@@ -937,6 +981,7 @@ class Family(ObjectType):
     id = ID()  # noqa: A003 (required for schema definition)
     name = String()
     meta = Field(NodeMeta)
+    runtime = Field(Runtime)
     depth = Int()
     proxies = graphene.List(
         lambda: FamilyProxy,
@@ -993,6 +1038,7 @@ class FamilyProxy(ObjectType):
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
         resolver=get_node_by_id)
+    runtime = Field(Runtime)
     state = String()
     states = graphene.List(String)
     state_totals = GenericScalar(resolver=resolve_state_totals)
@@ -1203,7 +1249,7 @@ async def mutator(
     root: Optional[Any],
     info: 'ResolveInfo',
     *,
-    command: str,
+    command: Optional[str] = None,
     workflows: Optional[List[str]] = None,
     exworkflows: Optional[List[str]] = None,
     **kwargs: Any
@@ -1218,11 +1264,13 @@ async def mutator(
         root: Parent field (if any) value object.
         info: GraphQL execution info.
         command: Mutation command name (name of method of
-            cylc.flow.network.resolvers.Resolvers or
-            Scheduler command_<name> method).
+            cylc.flow.network.resolvers.Resolvers or Scheduler command_<name>
+            method). If None, uses mutation class name converted to snake_case.
         workflows: List of workflow IDs.
         exworkflows: List of workflow IDs.
     """
+    if command is None:
+        command = to_snake_case(info.field_name)
     if workflows is None:
         workflows = []
     if exworkflows is None:
@@ -1304,6 +1352,11 @@ class BroadcastSetting(GenericScalar):
     """
 
 
+class BroadcastCyclePoint(graphene.String):
+    """A cycle point or `*`."""
+    # (broadcast supports either of those two but not cycle point globs)
+
+
 class TaskStatus(graphene.Enum):
     """The status of a task in a workflow."""
 
@@ -1437,7 +1490,7 @@ class Broadcast(Mutation):
             *not* clear all-cycle or all-namespace broadcasts.
 
         ''')
-        resolver = partial(mutator, command='broadcast')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1450,7 +1503,7 @@ class Broadcast(Mutation):
             required=True
         )
         cycle_points = graphene.List(
-            CyclePoint,
+            BroadcastCyclePoint,
             description=sstrip('''
                 List of cycle points to target (or `*` to cancel all all-cycle
                 broadcasts without canceling all specific-cycle broadcasts).
@@ -1496,7 +1549,7 @@ class SetHoldPoint(Mutation):
             Set workflow hold after cycle point. All tasks after this point
             will be held.
         ''')
-        resolver = partial(mutator, command='set_hold_point')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1513,9 +1566,9 @@ class Pause(Mutation):
         description = sstrip('''
             Pause a workflow.
 
-            This prevents submission of any task jobs.
+            This suspends submission of tasks.
         ''')
-        resolver = partial(mutator, command='pause')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1526,15 +1579,15 @@ class Pause(Mutation):
 class Message(Mutation):
     class Meta:
         description = sstrip('''
-            Record task job messages.
+            Record task messages.
 
-            Send task job messages to:
+            Send messages to:
             - The job stdout/stderr.
             - The job status file, if there is one.
             - The scheduler, if communication is possible.
 
-            Task jobs use this to record and report status such
-            as success and failure. Applications run by task jobs can use
+            Jobs use this to record and report status such
+            as success and failure. Applications run by jobs can use
             this command to report messages and to report registered task
             outputs.
         ''')
@@ -1560,7 +1613,7 @@ class ReleaseHoldPoint(Mutation):
 
             Held tasks do not submit their jobs even if ready to run.
         ''')
-        resolver = partial(mutator, command='release_hold_point')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1575,7 +1628,7 @@ class Resume(Mutation):
 
             See also the opposite command `pause`.
         ''')
-        resolver = partial(mutator, command='resume')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1618,7 +1671,7 @@ class SetVerbosity(Mutation):
             for example, if you choose `WARNING`, only warnings and critical
             messages will be logged.
         ''')
-        resolver = partial(mutator, command='set_verbosity')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1634,7 +1687,7 @@ class SetGraphWindowExtent(Mutation):
             of the data-store graph window.
 
         ''')
-        resolver = partial(mutator, command='set_graph_window_extent')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1659,7 +1712,7 @@ class Stop(Mutation):
             be executed prior to shutdown, unless
             the stop mode is `{WorkflowStopMode.Now.name}`.
         ''')
-        resolver = partial(mutator, command='stop')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1747,15 +1800,35 @@ class FlowMutationArguments:
             This should be a list of flow numbers OR a single-item list
             containing one of the following three strings:
 
-            Alternatively this may be a single-item list containing one of
-            the following values:
-
             * {FLOW_ALL} - Triggered tasks belong to all active flows
               (default).
             * {FLOW_NEW} - Triggered tasks are assigned to a new flow.
             * {FLOW_NONE} - Triggered tasks do not belong to any flow.
         ''')
     )
+    flow_wait = Boolean(
+        default_value=False,
+        description=sstrip('''
+            Should the workflow "wait" or "continue on" from this task?
+
+            If `false` the scheduler will spawn and run downstream tasks
+            as normal (default).
+
+            If `true` the scheduler will not spawn the downstream tasks
+            unless it has been caught by the same flow at a later time.
+
+            For example you might set this to True to trigger a task
+            ahead of a flow, where you don't want the scheduler to
+            "continue on" from this task until the flow has caught up
+            with it.
+        ''')
+    )
+    flow_descr = String(
+        description=sstrip('''
+            If starting a new flow, this field can be used to provide the
+            new flow with a description for later reference.
+        ''')
+    )
 
 
 class Hold(Mutation, TaskMutation):
@@ -1765,7 +1838,7 @@ class Hold(Mutation, TaskMutation):
 
             Held tasks do not submit their jobs even if ready to run.
         ''')
-        resolver = partial(mutator, command='hold')
+        resolver = mutator
 
 
 class Release(Mutation, TaskMutation):
@@ -1775,7 +1848,7 @@ class Release(Mutation, TaskMutation):
 
             See also the opposite command `hold`.
         ''')
-        resolver = partial(mutator, command='release')
+        resolver = mutator
 
 
 class Kill(Mutation, TaskMutation):
@@ -1790,7 +1863,7 @@ class Kill(Mutation, TaskMutation):
 class Poll(Mutation, TaskMutation):
     class Meta:
         description = sstrip('''
-            Poll (query) task jobs to verify and update their statuses.
+            Poll (query) jobs to verify and update their statuses.
 
             This checks the job status file and queries the
             job runner on the job platform.
@@ -1849,29 +1922,7 @@ class Trigger(Mutation, TaskMutation):
         resolver = partial(mutator, command='force_trigger_tasks')
 
     class Arguments(TaskMutation.Arguments, FlowMutationArguments):
-        flow_wait = Boolean(
-            default_value=False,
-            description=sstrip('''
-                Should the workflow "wait" or "continue on" from this task?
-
-                If `false` the scheduler will spawn and run downstream tasks
-                as normal (default).
-
-                If `true` the scheduler will not spawn the downstream tasks
-                unless it has been caught by the same flow at a later time.
-
-                For example you might set this to True to trigger a task
-                ahead of a flow, where you don't want the scheduled to
-                "continue on" from this task until the flow has caught up
-                with it.
-            ''')
-        )
-        flow_descr = String(
-            description=sstrip('''
-                If starting a new flow, this field can be used to provide the
-                new flow with a description for later reference.
-            ''')
-        )
+        ...
 
 
 def _mut_field(cls):
@@ -1987,7 +2038,7 @@ class Delta(Interface):
     )
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=Boolean(),
         delta_store=Boolean(default_value=True),
@@ -2062,7 +2113,7 @@ class Updated(ObjectType):
     )
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=Boolean(),
         delta_store=Boolean(default_value=True),
