@@ -16,7 +16,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Cylc configuration linter.
 
-Checks code style, deprecated syntax and other issues."""
+Checks code style, deprecated syntax and other issues.
+
+"""
 # NOTE: docstring needed for `cylc help all` output
 # (if editing check this still comes out as expected)
 
@@ -26,30 +28,35 @@ Check .cylc and .rc files for code style, deprecated syntax and other issues.
 
 By default, suggestions are written to stdout.
 
-In-place mode ( "-i, --inplace") writes suggestions into the file as comments.
+In-place mode ("-i, --inplace") writes suggestions into the file as comments.
 Commit to version control before using this, in case you want to back out.
 
+Configurations for Cylc lint can also be set in a pyproject.toml file.
+
 """
 from colorama import Fore
 from optparse import Values
 from pathlib import Path
 import re
-from typing import Generator
+import tomli
+from typing import Generator, Union
 
 from cylc.flow import LOG
+from cylc.flow.exceptions import CylcError
 from cylc.flow.option_parsers import (
     CylcOptionParser as COP,
+    WORKFLOW_ID_OR_PATH_ARG_DOC
 )
 from cylc.flow.cfgspec.workflow import upg, SPEC
+from cylc.flow.id_cli import parse_id
 from cylc.flow.parsec.config import ParsecConfig
 from cylc.flow.terminal import cli_function
-from cylc.flow.workflow_files import check_flow_file
 
 STYLE_GUIDE = (
-    'https://cylc.github.io/cylc-doc/latest/html/workflow-design-guide/'
+    'https://cylc.github.io/cylc-doc/stable/html/workflow-design-guide/'
     'style-guide.html#'
 )
-URL_STUB = "https://cylc.github.io/cylc-doc/latest/html/7-to-8/"
+URL_STUB = "https://cylc.github.io/cylc-doc/stable/html/7-to-8/"
 SECTION2 = r'\[\[\s*{}\s*\]\]'
 SECTION3 = r'\[\[\[\s*{}\s*\]\]\]'
 FILEGLOBS = ['*.rc', '*.cylc']
@@ -81,17 +88,20 @@ CHECKS_DESC = {'U': '7 to 8 upgrades', 'S': 'Style'}
 STYLE_CHECKS = {
     re.compile(r'^\t'): {
         'short': 'Use multiple spaces, not tabs',
-        'url': STYLE_GUIDE + 'tab-characters'
+        'url': STYLE_GUIDE + 'tab-characters',
+        'index': 1
     },
     # Not a full test, but if a non section is not indented...
     re.compile(r'^[^\{\[|\s]'): {
         'short': 'Item not indented.',
-        'url': STYLE_GUIDE + 'indentation'
+        'url': STYLE_GUIDE + 'indentation',
+        'index': 2
     },
     #            [section]
     re.compile(r'^\s+\[[^\[.]*\]'): {
         'short': 'Top level sections should not be indented.',
-        'url': STYLE_GUIDE + 'indentation'
+        'url': STYLE_GUIDE + 'indentation',
+        'index': 3
     },
     # 2 or 4 space indentation both seem reasonable:
     re.compile(r'^(|\s|\s{2,3}|\s{5,})\[\[[^\[.]*\]\]'): {
@@ -99,28 +109,32 @@ STYLE_CHECKS = {
             'Second level sections should be indented exactly '
             '4 spaces.'
         ),
-        'url': STYLE_GUIDE + 'indentation'
+        'url': STYLE_GUIDE + 'indentation',
+        'index': 4
     },
     re.compile(r'^(|\s{1,7}|\s{9,})\[\[\[[^\[.]*\]\]\]'): {
         'short': (
             'Third level sections should be indented exactly '
             '8 spaces.'
         ),
-        'url': STYLE_GUIDE + 'indentation'
+        'url': STYLE_GUIDE + 'indentation',
+        'index': 5
     },
     re.compile(r'\s$'): {
         'short': 'trailing whitespace.',
-        'url': STYLE_GUIDE + 'trailing-whitespace'
+        'url': STYLE_GUIDE + 'trailing-whitespace',
+        'index': 6
     },
-    # Consider re-adding this as an option later:
-    # re.compile(r'^.{80,}'): {
-    #     'short': 'line > 79 characters.',
-    #     'url': STYLE_GUIDE + 'line-length-and-continuation'
-    # },
     re.compile(r'inherit\s*=\s*[a-z].*$'): {
         'short': 'Family name contains lowercase characters.',
-        'url': STYLE_GUIDE + 'task-naming-conventions'
+        'url': STYLE_GUIDE + 'task-naming-conventions',
+        'index': 7
     },
+    # re.compile(r'^.{{maxlen},}'): {
+    #     'short': 'line > {maxlen} characters.',
+    #     'url': STYLE_GUIDE + 'line-length-and-continuation',
+    #     'index': 8
+    # },
 }
 # Subset of deprecations which are tricky (impossible?) to scrape from the
 # upgrader.
@@ -157,6 +171,143 @@ MANUAL_DEPRECATIONS = {
         )
     },
 }
+RULESETS = ['728', 'style', 'all']
+EXTRA_TOML_VALIDATION = {
+    'ignore': {
+        lambda x: re.match(r'[A-Z]\d\d\d', x):
+            '{item} not valid: Ignore codes should be in the form X001',
+        lambda x: x in [
+            f'{i["purpose"]}{i["index"]:03d}'
+            for i in parse_checks(['728', 'style']).values()
+        ]:
+            '{item} is a not a known linter code.'
+    },
+    'rulesets': {
+        lambda item: item in RULESETS:
+            '{item} not valid: Rulesets can be '
+            '\'728\', \'style\' or \'all\'.'
+    },
+    'max-line-length': {
+        lambda x: isinstance(x, int):
+            'max-line-length must be an integer.'
+    },
+    # consider checking that item is file?
+    'exclude': {}
+}
+
+
+def validate_toml_items(tomldata):
+    """Check that all tomldata items are lists of strings
+
+    Explicitly checks and raises if tomldata not:
+        - str in EXTRA_TOML_VALIDATION.keys()
+        - item is not a list of strings.
+
+    Plus additional validation for each set of values.
+    """
+    for key, items in tomldata.items():
+        # Key should only be one of the allowed keys:
+        if key not in EXTRA_TOML_VALIDATION.keys():
+            raise CylcError(
+                f'Only {[*EXTRA_TOML_VALIDATION.keys()]} '
+                f'allowed as toml sections but you used {key}'
+            )
+        if key != 'max-line-length':
+            # Item should be a list...
+            if not isinstance(items, list):
+                raise CylcError(
+                    f'{key} should be a list, but was: {items}')
+            # ... of strings
+            for item in items:
+                if not isinstance(item, str):
+                    raise CylcError(
+                        f'Config {item} should be a string but '
+                        f'is {str(type(item))}'
+                    )
+                for check, message in EXTRA_TOML_VALIDATION[key].items():
+                    if not check(item):
+                        raise CylcError(
+                            message.format(item=item)
+                        )
+    return True
+
+
+def get_pyproject_toml(dir_):
+    """if a pyproject.toml file is present open it and return settings.
+    """
+    keys = ['rulesets', 'ignore', 'exclude', 'max-line-length']
+    tomlfile = Path(dir_ / 'pyproject.toml')
+    tomldata = {}
+    if tomlfile.is_file():
+        try:
+            loadeddata = tomli.loads(tomlfile.read_text())
+        except tomli.TOMLDecodeError as exc:
+            raise CylcError(f'pyproject.toml did not load: {exc}')
+
+        if any(
+            [i in loadeddata for i in ['cylc-lint', 'cylclint', 'cylc_lint']]
+        ):
+            for key in keys:
+                tomldata[key] = loadeddata.get('cylc-lint').get(key, [])
+            validate_toml_items(tomldata)
+    if not tomldata:
+        tomldata = {key: [] for key in keys}
+    return tomldata
+
+
+def merge_cli_with_tomldata(
+    clidata, tomldata,
+    override_cli_default_rules=False
+):
+    """Merge options set by pyproject.toml with CLI options.
+
+    rulesets: CLI should override toml.
+    ignore: CLI and toml should be combined with no duplicates.
+    exclude: No CLI equivalent, return toml if any.
+
+    Args:
+        override_cli_default_rules: If user doesn't specifiy a ruleset use the
+            rules from the tomlfile - i.e: if we've set 'rulesets': 'style'
+            we probably don't want to get warnings about 728 upgrades by
+            default, but only if we ask for it on the CLI.
+
+    Examples:
+    >>> result = merge_cli_with_tomldata(
+    ... {'rulesets': ['foo'], 'ignore': ['R101'], 'exclude': []},
+    ... {'rulesets': ['bar'], 'ignore': ['R100'], 'exclude': ['*.bk']})
+    >>> result['ignore']
+    ['R100', 'R101']
+    >>> result['rulesets']
+    ['foo']
+    >>> result['exclude']
+    ['*.bk']
+    """
+    if isinstance(clidata['rulesets'][0], list):
+        clidata['rulesets'] = clidata['rulesets'][0]
+
+    output = {}
+
+    # Combine 'ignore' sections:
+    output['ignore'] = sorted(set(clidata['ignore'] + tomldata['ignore']))
+
+    # Replace 'rulesets from toml with those from CLI if they exist:
+
+    if override_cli_default_rules:
+        output['rulesets'] = (
+            tomldata['rulesets'] if tomldata['rulesets']
+            else clidata['rulesets']
+        )
+    else:
+        output['rulesets'] = (
+            clidata['rulesets'] if clidata['rulesets']
+            else tomldata['rulesets']
+        )
+
+    # Return 'exclude' and 'max-line-length' for the tomldata:
+    output['exclude'] = tomldata['exclude']
+    output['max-line-length'] = tomldata.get('max-line-length', None)
+
+    return output
 
 
 def list_to_config(path_, is_section=False):
@@ -235,44 +386,64 @@ def get_upgrader_info():
     return deprecations
 
 
-def get_checkset_from_name(name):
-    """Take a ruleset name and return a ruleset code
+PURPOSE_FILTER_MAP = {
+    'style': 'S',
+    '728': 'U',
+}
 
-    Examples:
-        >>> get_checkset_from_name('728')
-        ['U']
-        >>> get_checkset_from_name('style')
-        ['S']
-        >>> get_checkset_from_name('all')
-        ['U', 'S']
-    """
-    if name == '728':
-        purpose_filters = ['U']
-    elif name == 'style':
-        purpose_filters = ['S']
-    else:
-        purpose_filters = ['U', 'S']
-    return purpose_filters
 
+def parse_checks(check_args, ignores=None, max_line_len=None, reference=False):
+    """Prepare dictionary of checks.
 
-def parse_checks(check_arg):
-    """Collapse metadata in checks dicts.
+    Args:
+        check_arg: list of types of checks to run,
+            currently expecting '728' and/or 'style'
+        ignores: list of codes to ignore.
+        max_line_len: Adds a specific style warning for lines longer than
+            this. (If None, rule not enforced)
+        reference: Function is being used to get a reference. If true
+            max-line-length will have a generic message, rather than
+            using any specific value.
     """
+    ignores = ignores or []
     parsedchecks = {}
-    purpose_filters = get_checkset_from_name(check_arg)
+    purpose_filters = [PURPOSE_FILTER_MAP[i] for i in check_args]
 
     checks = {'U': get_upgrader_info(), 'S': STYLE_CHECKS}
 
     for purpose, ruleset in checks.items():
         if purpose in purpose_filters:
+            # Run through the rest of the config items.
             for index, (pattern, meta) in enumerate(ruleset.items(), start=1):
                 meta.update({'purpose': purpose})
-                meta.update({'index': index})
-                parsedchecks.update({pattern: meta})
+                if 'index' not in meta:
+                    meta.update({'index': index})
+                if f'{purpose}{index:03d}' not in ignores:
+                    parsedchecks.update({pattern: meta})
+            if 'S' in purpose and "S008" not in ignores:
+                if not max_line_len:
+                    max_line_len = 130
+                regex = r"^.{" + str(max_line_len) + r"}"
+                if reference:
+                    msg = (
+                        'line > ``<max_line_len>`` characters. Max line '
+                        ' length set in pyproject.toml (default 130)'
+                    )
+                else:
+                    msg = f'line > {max_line_len} characters.'
+                parsedchecks[re.compile(regex)] = {
+                    'short': msg,
+                    'url': STYLE_GUIDE + 'line-length-and-continuation',
+                    'index': 8,
+                    'purpose': 'S'
+                }
     return parsedchecks
 
 
-def check_cylc_file(dir_, file_, checks, modify=False):
+def check_cylc_file(
+    dir_, file_, checks,
+    modify=False,
+):
     """Check A Cylc File for Cylc 7 Config"""
     file_rel = file_.relative_to(dir_)
     # Set mode as read-write or read only.
@@ -318,12 +489,20 @@ def check_cylc_file(dir_, file_, checks, modify=False):
     return count
 
 
-def get_cylc_files(base: Path) -> Generator[Path, None, None]:
+def get_cylc_files(
+    base: Path, exclusions: Union[list, None] = None
+) -> Generator[Path, None, None]:
     """Given a directory yield paths to check."""
+    exclusions = [] if exclusions is None else exclusions
+    except_these_files = [
+        file for exclusion in exclusions for file in base.rglob(exclusion)]
     for rglob in FILEGLOBS:
         for path in base.rglob(rglob):
             # Exclude log directory:
-            if path.relative_to(base).parts[0] != 'log':
+            if (
+                path.relative_to(base).parts[0] != 'log'
+                and path not in except_these_files
+            ):
                 yield path
 
 
@@ -403,8 +582,11 @@ def get_reference_text(checks):
 def get_option_parser() -> COP:
     parser = COP(
         COP_DOC,
-        argdoc=[COP.optional(('DIR ...', 'Directories to lint'))],
+        argdoc=[
+            COP.optional(WORKFLOW_ID_OR_PATH_ARG_DOC)
+        ],
     )
+
     parser.add_option(
         '--inplace', '-i',
         help=(
@@ -420,8 +602,8 @@ def get_option_parser() -> COP:
             'Set of rules to use: '
             '("728", "style", "all")'
         ),
-        default='all',
-        choices=('728', 'style', 'all'),
+        default='',
+        choices=["728", "style", "all", ''],
         dest='linter'
     )
     parser.add_option(
@@ -433,79 +615,114 @@ def get_option_parser() -> COP:
         default=False,
         dest='ref_mode'
     )
+    parser.add_option(
+        '--ignore', '-n',
+        help=(
+            'Ignore this check number.'
+        ),
+        action='append',
+        default=[],
+        dest='ignores',
+        metavar="CODE",
+        choices=tuple([f'S{i["index"]:03d}' for i in STYLE_CHECKS.values()])
+    )
 
     return parser
 
 
 @cli_function(get_option_parser)
-def main(parser: COP, options: 'Values', *targets) -> None:
+def main(parser: COP, options: 'Values', target=None) -> None:
     if options.ref_mode:
-        print(get_reference_text(parse_checks(options.linter)))
+        if options.linter in {'all', ''}:
+            rulesets = ['728', 'style']
+        else:
+            rulesets = [options.linter]
+        print(get_reference_text(parse_checks(rulesets, reference=True)))
         exit(0)
 
-    # Expand paths so that message will return full path
-    # & ensure that CWD is used if no path is given:
-    if targets:
-        targets = tuple(Path(path).resolve() for path in targets)
-    else:
-        targets = (str(Path.cwd()),)
+    # If target not given assume we are looking at PWD
+    if target is None:
+        target = str(Path.cwd())
 
-    # make sure the targets are all src/run directories
-    for target in targets:
-        check_flow_file(target)
+    # make sure the target is a src/run directories
+    _, _, target = parse_id(
+        target,
+        src=True,
+        constraint='workflows',
+    )
 
     # Get a list of checks bas ed on the checking options:
     # Allow us to check any number of folders at once
-    for target in targets:
-        count = 0
-        target = Path(target)
-        if not target.exists():
-            LOG.warn(f'Path {target} does not exist.')
-        else:
-            # Check whether target is an upgraded Cylc 8 workflow.
-            # If it isn't then we shouldn't run the 7-to-8 checks upon
-            # it:
-            cylc8 = (target / 'flow.cylc').exists()
-            if not cylc8 and options.linter == '728':
-                LOG.error(
-                    f'{target} not a Cylc 8 workflow: '
-                    'Lint after renaming '
-                    '"suite.rc" to "flow.cylc"'
-                )
-                continue
-            elif not cylc8 and options.linter == 'all':
-                check_names = 'style'
-            else:
-                check_names = options.linter
+    count = 0
+    target = target.parent
+    ruleset_default = False
+    if options.linter == 'all':
+        options.linter = ['728', 'style']
+    elif options.linter == '':
+        options.linter = ['728', 'style']
+        ruleset_default = True
+    else:
+        options.linter = [options.linter]
+    tomlopts = get_pyproject_toml(target)
+    mergedopts = merge_cli_with_tomldata(
+        {
+            'exclude': [],
+            'ignore': options.ignores,
+            'rulesets': options.linter
+        },
+        tomlopts,
+        ruleset_default
+    )
 
-            # Check each file:
-            checks = parse_checks(check_names)
-            for file_ in get_cylc_files(target):
-                LOG.debug(f'Checking {file_}')
-                count += check_cylc_file(
-                    target,
-                    file_,
-                    checks,
-                    options.inplace,
-                )
+    # Check whether target is an upgraded Cylc 8 workflow.
+    # If it isn't then we shouldn't run the 7-to-8 checks upon
+    # it:
+    cylc8 = (target / 'flow.cylc').exists()
+    if not cylc8 and mergedopts['rulesets'] == ['728']:
+        LOG.error(
+            f'{target} not a Cylc 8 workflow: '
+            'Lint after renaming '
+            '"suite.rc" to "flow.cylc"'
+        )
+        exit(0)
+    elif not cylc8 and '728' in mergedopts['rulesets']:
+        check_names = mergedopts['rulesets']
+        check_names.remove('728')
+    else:
+        check_names = mergedopts['rulesets']
 
-        if count > 0:
-            msg = (
-                f'\n{Fore.YELLOW}'
-                f'Checked {target} against {check_names} '
-                f'rules and found {count} issue'
-                f'{"s" if count > 1 else ""}.'
-            )
-        else:
-            msg = (
-                f'{Fore.GREEN}'
-                f'Checked {target} against {check_names} rules and '
-                'found no issues.'
-            )
+    # Check each file:
+    checks = parse_checks(
+        check_names,
+        ignores=mergedopts['ignore'],
+        max_line_len=mergedopts['max-line-length']
+    )
+    for file_ in get_cylc_files(target, mergedopts['exclude']):
+        LOG.debug(f'Checking {file_}')
+        count += check_cylc_file(
+            target,
+            file_,
+            checks,
+            options.inplace,
+        )
 
-        print(msg)
+    if count > 0:
+        msg = (
+            f'\n{Fore.YELLOW}'
+            f'Checked {target} against {check_names} '
+            f'rules and found {count} issue'
+            f'{"s" if count > 1 else ""}.'
+        )
+    else:
+        msg = (
+            f'{Fore.GREEN}'
+            f'Checked {target} against {check_names} rules and '
+            'found no issues.'
+        )
+
+    print(msg)
 
 
 # NOTE: use += so that this works with __import__
 # (docstring needed for `cylc help all` output)
-__doc__ += get_reference_rst(parse_checks('all'))
+__doc__ += get_reference_rst(parse_checks(['728', 'style'], reference=True))
