@@ -29,6 +29,7 @@ import re
 from subprocess import Popen, PIPE, DEVNULL, TimeoutExpired
 import shlex
 import shutil
+import sqlite3
 from time import sleep
 from typing import (
     Any, Container, Deque, Dict, Iterable, List, NamedTuple, Optional, Set,
@@ -78,7 +79,7 @@ from cylc.flow.hostuserutil import (
 )
 from cylc.flow.remote import (
     DEFAULT_RSYNC_OPTS,
-    _construct_ssh_cmd,
+    construct_cylc_server_ssh_cmd,
     construct_ssh_cmd,
 )
 from cylc.flow.terminal import parse_dirty_json
@@ -190,8 +191,17 @@ class WorkflowFiles:
     RUN_N = 'runN'
     """Symbolic link for latest run"""
 
-    LOG_DIR = 'log'
-    """Workflow log directory."""
+    class LogDir:
+        """The directory containing workflow log files"""
+
+        DIRNAME = 'log'
+        """Workflow log directory."""
+
+        INSTALL = 'install'
+        """Install log dir"""
+
+        VERSION = 'version'
+        """Version control log dir"""
 
     SHARE_DIR = 'share'
     """Workflow share directory."""
@@ -240,7 +250,8 @@ class WorkflowFiles:
         """Symlink to the workflow definition (For run dir)."""
 
     RESERVED_DIRNAMES = frozenset([
-        LOG_DIR, SHARE_DIR, WORK_DIR, RUN_N, Service.DIRNAME, Install.DIRNAME
+        LogDir.DIRNAME, SHARE_DIR, WORK_DIR, RUN_N,
+        Service.DIRNAME, Install.DIRNAME
     ])
     """Reserved directory names that cannot be present in a source dir."""
 
@@ -248,7 +259,7 @@ class WorkflowFiles:
     """Reserved filenames that cannot be used as run names."""
 
     SYMLINK_DIRS = frozenset([
-        SHARE_CYCLE_DIR, SHARE_DIR, LOG_DIR, WORK_DIR, ''
+        SHARE_CYCLE_DIR, SHARE_DIR, LogDir.DIRNAME, WORK_DIR, ''
     ])
     """The paths of the symlink dirs that may be set in
     global.cylc[install][symlink dirs], relative to the run dir
@@ -284,7 +295,8 @@ class ContactFileFields:
     """The process ID of the running workflow on ``CYLC_WORKFLOW_HOST``."""
 
     COMMAND = 'CYLC_WORKFLOW_COMMAND'
-    """The command that was used to run the workflow on ``CYLC_WORKFLOW_HOST```.
+    """The command that was used to run the workflow on
+    ``CYLC_WORKFLOW_HOST```.
 
     Note that this command may be affected by:
 
@@ -405,7 +417,7 @@ def _is_process_running(
     metric = f'[["Process", {pid}]]'
     if is_remote_host(host):
         cmd = ['psutil']
-        cmd = _construct_ssh_cmd(cmd, host)
+        cmd = construct_cylc_server_ssh_cmd(cmd, host)
     else:
         cmd = ['cylc', 'psutil']
     proc = Popen(  # nosec
@@ -452,7 +464,9 @@ def _is_process_running(
     return cli_format(process['cmdline']) == command
 
 
-def detect_old_contact_file(reg: str, contact_data=None) -> None:
+def detect_old_contact_file(
+    reg: str, contact_data=None
+) -> None:
     """Check if the workflow process is still running.
 
     As a side-effect this should detect and rectify the situation
@@ -468,6 +482,7 @@ def detect_old_contact_file(reg: str, contact_data=None) -> None:
 
     Args:
         reg: workflow name
+        contact_date:
 
     Raises:
         CylcError:
@@ -759,21 +774,22 @@ def _clean_check(opts: 'Values', reg: str, run_dir: Path) -> None:
         )
 
 
-def init_clean(reg: str, opts: 'Values') -> None:
+def init_clean(id_: str, opts: 'Values') -> None:
     """Initiate the process of removing a stopped workflow from the local
     scheduler filesystem and remote hosts.
 
     Args:
-        reg: Workflow name/ID.
+        id_: Workflow ID.
         opts: CLI options object for cylc clean.
+
     """
-    local_run_dir = Path(get_workflow_run_dir(reg))
+    local_run_dir = Path(get_workflow_run_dir(id_))
     with suppress(InputError):
-        local_run_dir, reg = infer_latest_run(
+        local_run_dir, id_ = infer_latest_run(
             local_run_dir, implicit_runN=False, warn_runN=False
         )
     try:
-        _clean_check(opts, reg, local_run_dir)
+        _clean_check(opts, id_, local_run_dir)
     except FileNotFoundError as exc:
         LOG.info(exc)
         return
@@ -783,31 +799,46 @@ def init_clean(reg: str, opts: 'Values') -> None:
 
     if not opts.local_only:
         platform_names = None
-        try:
-            platform_names = get_platforms_from_db(local_run_dir)
-        except FileNotFoundError:
+        db_file = Path(get_workflow_srv_dir(id_), 'db')
+        if not db_file.is_file():
+            # no DB -> do nothing
             if opts.remote_only:
                 raise ServiceFileError(
-                    f"No workflow database for {reg} - cannot perform "
+                    f"No workflow database for {id_} - cannot perform "
                     "remote clean"
                 )
             LOG.info(
-                f"No workflow database for {reg} - will only clean locally"
+                f"No workflow database for {id_} - will only clean locally"
             )
-        except ServiceFileError as exc:
-            raise ServiceFileError(f"Cannot clean {reg} - {exc}")
+        else:
+            # DB present -> load platforms
+            try:
+                platform_names = get_platforms_from_db(local_run_dir)
+            except ServiceFileError as exc:
+                raise ServiceFileError(f"Cannot clean {id_} - {exc}")
+            except sqlite3.OperationalError as exc:
+                # something went wrong with the query
+                # e.g. the table/field we need isn't there
+                LOG.warning(
+                    'This database is either corrupted or not compatible with'
+                    ' this version of "cylc clean".'
+                    '\nTry using the version of Cylc the workflow was last ran'
+                    ' with to remove it.'
+                    '\nOtherwise please delete the database file.'
+                )
+                raise ServiceFileError(f"Cannot clean {id_} - {exc}")
 
         if platform_names and platform_names != {'localhost'}:
             remote_clean(
-                reg, platform_names, opts.rm_dirs, opts.remote_timeout
+                id_, platform_names, opts.rm_dirs, opts.remote_timeout
             )
 
     if not opts.remote_only:
         # Must be after remote clean
-        clean(reg, local_run_dir, rm_dirs)
+        clean(id_, local_run_dir, rm_dirs)
 
 
-def clean(reg: str, run_dir: Path, rm_dirs: Optional[Set[str]] = None) -> None:
+def clean(id_: str, run_dir: Path, rm_dirs: Optional[Set[str]] = None) -> None:
     """Remove a stopped workflow from the local filesystem only.
 
     Deletes the workflow run directory and any symlink dirs, or just the
@@ -817,11 +848,12 @@ def clean(reg: str, run_dir: Path, rm_dirs: Optional[Set[str]] = None) -> None:
     possible to clean any symlink dirs.
 
     Args:
-        reg: Workflow name.
+        id_: Workflow ID.
         run_dir: Absolute path of the workflow's run dir.
         rm_dirs: Set of sub dirs to remove instead of the whole run dir.
+
     """
-    symlink_dirs = get_symlink_dirs(reg, run_dir)
+    symlink_dirs = get_symlink_dirs(id_, run_dir)
     if rm_dirs is not None:
         # Targeted clean
         for pattern in rm_dirs:
@@ -830,18 +862,13 @@ def clean(reg: str, run_dir: Path, rm_dirs: Optional[Set[str]] = None) -> None:
         # Wholesale clean
         LOG.debug(f"Cleaning {run_dir}")
         for symlink in symlink_dirs:
-            # Remove <symlink_dir>/cylc-run/<reg>/<symlink>
+            # Remove <symlink_dir>/cylc-run/<id>/<symlink>
             remove_dir_and_target(run_dir / symlink)
         if '' not in symlink_dirs:
             # if run dir isn't a symlink dir and hasn't been deleted yet
             remove_dir_and_target(run_dir)
-    # Tidy up if necessary
-    # Remove any empty parents of run dir up to ~/cylc-run/
-    remove_empty_parents(run_dir, reg)
-    for symlink, target in symlink_dirs.items():
-        # Remove empty parents of symlink target up to <symlink_dir>/cylc-run/
-        remove_empty_parents(target, Path(reg, symlink))
 
+    # Tidy up if necessary
     # Remove `runN` symlink if it's now broken
     runN = run_dir.parent / WorkflowFiles.RUN_N
     if (
@@ -850,6 +877,20 @@ def clean(reg: str, run_dir: Path, rm_dirs: Optional[Set[str]] = None) -> None:
         os.readlink(str(runN)) == run_dir.name
     ):
         runN.unlink()
+    # Remove _cylc-install if it's the only thing left
+    cylc_install_dir = run_dir.parent / WorkflowFiles.Install.DIRNAME
+    for entry in run_dir.parent.iterdir():
+        if entry == cylc_install_dir:
+            continue
+        break
+    else:  # no break
+        if cylc_install_dir.is_dir():
+            remove_dir_or_file(cylc_install_dir)
+    # Remove any empty parents of run dir up to ~/cylc-run/
+    remove_empty_parents(run_dir, id_)
+    for symlink, target in symlink_dirs.items():
+        # Remove empty parents of symlink target up to <symlink_dir>/cylc-run/
+        remove_empty_parents(target, Path(id_, symlink))
 
 
 def get_symlink_dirs(reg: str, run_dir: Union[Path, str]) -> Dict[str, Path]:
@@ -1067,6 +1108,10 @@ def _remote_clean_cmd(
         platform: Config for the platform on which to remove the workflow.
         rm_dirs: Sub dirs to remove instead of the whole run dir.
         timeout: Number of seconds to wait before cancelling the command.
+
+    Raises:
+        NoHostsError: If the platform is not contactable.
+
     """
     LOG.debug(
         f"Cleaning {reg} on install target: {platform['install target']} "
@@ -1077,9 +1122,11 @@ def _remote_clean_cmd(
         for item in rm_dirs:
             cmd.extend(['--rm', item])
     cmd = construct_ssh_cmd(
-        cmd, platform,
+        cmd,
+        platform,
         get_host_from_platform(platform),
-        timeout=timeout, set_verbosity=True
+        timeout=timeout,
+        set_verbosity=True,
     )
     LOG.debug(" ".join(cmd))
     return Popen(  # nosec
@@ -1154,21 +1201,31 @@ def get_workflow_title(reg):
 
 
 def get_platforms_from_db(run_dir):
-    """Load the set of names of platforms (that jobs ran on) from the
-    workflow database.
+    """Return the set of names of platforms (that jobs ran on) from the DB.
+
+    Warning:
+        This does NOT upgrade the workflow database!
+
+        We could upgrade the DB for backward compatiblity, but we haven't
+        got any upgraders for this table yet so there's no point.
+
+        Note that upgrading the DB here would not help with forward
+        compatibility. We can't apply upgraders which don't exist yet.
 
     Args:
         run_dir (str): The workflow run directory.
+
+    Raises:
+        sqlite3.OperationalError: in the event the table/field required for
+        cleaning is not present.
+
     """
     workflow_db_mgr = WorkflowDatabaseManager(
         os.path.join(run_dir, WorkflowFiles.Service.DIRNAME))
-    workflow_db_mgr.check_workflow_db_compatibility()
-    try:
-        pri_dao = workflow_db_mgr.get_pri_dao()
+    with workflow_db_mgr.get_pri_dao() as pri_dao:
         platform_names = pri_dao.select_task_job_platforms()
-        return platform_names
-    finally:
-        pri_dao.close()
+
+    return platform_names
 
 
 def check_deprecation(path, warn=True):
@@ -1413,15 +1470,10 @@ def _open_install_log(rund, logger):
     rund = Path(rund).expanduser()
     log_type = logger.name[logger.name.startswith('cylc-') and len('cylc-'):]
     log_dir = Path(
-        rund,
-        WorkflowFiles.LOG_DIR,
-        'install')
+        rund, WorkflowFiles.LogDir.DIRNAME, WorkflowFiles.LogDir.INSTALL)
     log_files = get_sorted_logs_by_time(log_dir, '*.log')
-    if log_files:
-        log_num = get_next_log_number(log_files[-1])
-    else:
-        log_num = '01'
-    log_path = Path(log_dir, f"{log_num}-{log_type}.log")
+    log_num = get_next_log_number(log_files[-1]) if log_files else 1
+    log_path = Path(log_dir, f"{log_num:02d}-{log_type}.log")
     log_parent_dir = log_path.parent
     log_parent_dir.mkdir(exist_ok=True, parents=True)
     handler = logging.FileHandler(log_path)
@@ -1455,23 +1507,35 @@ def get_rsync_rund_cmd(src, dst, reinstall=False, dry_run=False):
         rsync_cmd.append("--dry-run")
     if reinstall:
         rsync_cmd.append('--delete')
-    for exclude in [
+
+    exclusions = [
         '.git',
         '.svn',
         '.cylcignore',
-        'rose-suite.conf',
         'opt/rose-suite-cylc-install.conf',
-        WorkflowFiles.LOG_DIR,
+        WorkflowFiles.LogDir.DIRNAME,
         WorkflowFiles.WORK_DIR,
         WorkflowFiles.SHARE_DIR,
         WorkflowFiles.Install.DIRNAME,
         WorkflowFiles.Service.DIRNAME
-    ]:
-        if (Path(src).joinpath(exclude).exists() or
-                Path(dst).joinpath(exclude).exists()):
-            rsync_cmd.append(f"--exclude={exclude}")
-    if Path(src).joinpath('.cylcignore').exists():
-        rsync_cmd.append("--exclude-from=.cylcignore")
+    ]
+
+    # This is a hack to make sure that changes to rose-suite.conf
+    # are considered when re-installing.
+    # It should be removed after https://github.com/cylc/cylc-rose/issues/149
+    if not dry_run:
+        exclusions.append('rose-suite.conf')
+
+    for exclude in exclusions:
+        if (
+            Path(src).joinpath(exclude).exists() or
+            Path(dst).joinpath(exclude).exists()
+        ):
+            # Note '/' is the rsync "anchor" to the top level:
+            rsync_cmd.append(f"--exclude=/{exclude}")
+    cylcignore_file = Path(src).joinpath('.cylcignore')
+    if cylcignore_file.exists():
+        rsync_cmd.append(f"--exclude-from={cylcignore_file}")
     rsync_cmd.append(f"{src}/")
     rsync_cmd.append(f"{dst}/")
 
@@ -1517,12 +1581,19 @@ def reinstall_workflow(
         reinstall=True,
         dry_run=dry_run,
     )
+
+    # Add '+++' to -out-format to mark lines passed through formatter.
+    rsync_cmd.append('--out-format=+++%o %n%L+++')
+
+    # Run rsync command:
     reinstall_log.info(cli_format(rsync_cmd))
     LOG.debug(cli_format(rsync_cmd))
     proc = Popen(rsync_cmd, stdout=PIPE, stderr=PIPE, text=True)  # nosec
     # * command is constructed via internal interface
     stdout, stderr = proc.communicate()
-    stdout = stdout.strip()
+
+    # Strip unwanted output.
+    stdout = ('\n'.join(re.findall(r'\+\+\+(.*)\+\+\+', stdout))).strip()
     stderr = stderr.strip()
 
     if proc.returncode != 0:
@@ -1736,7 +1807,7 @@ def detect_both_flow_and_suite(path: Path) -> None:
     msg = (f"Both {WorkflowFiles.FLOW_FILE} and {WorkflowFiles.SUITE_RC} "
            f"files are present in {path}. Please remove one and"
            " try again. For more information visit: https://cylc.github.io/"
-           "cylc-doc/latest/html/7-to-8/summary.html#backward-compatibility")
+           "cylc-doc/stable/html/7-to-8/summary.html#backward-compatibility")
     if path.resolve().name == WorkflowFiles.SUITE_RC:
         flow_cylc = path.parent / WorkflowFiles.FLOW_FILE
     elif (path / WorkflowFiles.SUITE_RC).is_file():
@@ -1833,11 +1904,11 @@ def validate_source_dir(
             If log, share, work or _cylc-install directories exist in the
             source directory.
     """
-    # Ensure source dir does not contain log, share, work, _cylc-install
+    # Source dir must not contain reserved run dir names (as file or dir).
     for dir_ in WorkflowFiles.RESERVED_DIRNAMES:
-        if Path(source, dir_).is_dir():
+        if Path(source, dir_).exists():
             raise WorkflowFilesError(
-                f"{workflow_name} installation failed. "
+                f"{workflow_name} installation failed "
                 f"- {dir_} exists in source directory."
             )
     check_flow_file(source)
